/*****************************************************************

 Title: A Standard Macro Collection for Table 1
 Author: Kengo NAGASHIMA
 Version: 4.0.0
 Last Updated Date: 2018/12/03
 Recommended Environment: SAS 9.4 and maybe later

 - 4.0.0: a public version
 - 3.0.6: %contTableMake, fix for MEAN
 - 3.0.5: %contTableMake, fix for IQR
 - 3.0.4: %tableInit, fix for style templates
 - 3.0.3: %contTableMake, fix for printing N
 - 3.0.2: %discTableMake, fix for only 1 category data

*****************************************************************/


/* global macro variables */
%global data out group pvalue color;
option spool;


%macro tableInit(
  nprint = 1, footer = 1, font = "Times New Roman",
  hlabel = "Characteristic", slabel = "Study population"
);
ods graphics off;
ods escapechar = '^';

/* initialize color counter */
%if "&color." = "0" %then %let color = 2;
%else %let color = 0;


/* initialize output data */
proc datasets lib = work;
  delete &out.;
run; quit;


/* group variable setting */
%if "&group." = "" %then %do;
data _null_;
  set &data.;
  call symput("K", 1);
  %if "&nprint." = "1" %then
    call symput("label1", cat(&slabel., "/(^{style[font_style=Italic]n} = ", _N_, ")"));
  %else
    call symput("label1", &slabel.);;
run;
%end;
%else %do;
proc freq data = &data. noprint;
  table &group. / out = _freq_;
data _null_;
  set _freq_;
  call symput("K", compress(_N_));
run;
data _null_;
  set sashelp.vcolumn;
  if upcase(libname) = "WORK" &
     upcase(memtype) = "DATA" &
     upcase(memname) = upcase("&data.") &
     upcase(name) = upcase("&group.")
  then call symput("gfmt", trim(format));
run;
data _null_;
  set _freq_;
  %do i = 1 %to &K.;
    %if "&gfmt." = " " %then %do;
      %if "&nprint." = "1" %then
        if _N_ = &i. then
          call symput("label&i.", cat(&group., "/(^{style[font_style=Italic]n} = ", COUNT, ")"));
      %else
        if _N_ = &i. then
          call symput("label&i.", &group.);;
    %end;
    %else %do;
      %if "&nprint." = "1" %then
        if _N_ = &i. then
          call symput("label&i.", cat(put(&group., &gfmt.), "/(^{style[font_style=Italic]n} = ", COUNT, ")"));
      %else
        if _N_ = &i. then
          call symput("label&i.", put(&group., &gfmt.));;
    %end;
  %end;
run;
%end;


/* fotter */
%if "&footer." ^= "0" %then %do;
data _null_;
  call symput("footLog", cat(put(date(), yymmdd10.), " ", put(time(), time.)));
run;
%end;


/* template gen */
proc template;
define style Styles.MyJournal;
  parent = styles.Journal;
  style fonts /
    'docFont' = (&font., 3)
    'TitleFont' = (&font., 3, bold)
    'StrongFont' = (&font., 3)
    'EmphasisFont' = (&font., 3)
    'HeadingFont' = (&font., 3)
    'FixedEmphasisFont' = (&font., 3)
    'FixedFont' = (&font., 3)
    'TitleFont2' = (&font., 3)
    'BatchFixedFont' = (&font., 3)
    'FixedStrongFont' = (&font., 3);
end;
define table &out.tmpl;
  column
    name 
    %do i = 1 %to &K.;
      c&i. p&i. 
    %end;
    %if "&K." ^= "1" & "&pvalue." = "1" %then %do;
      pvalue 
    %end;
    col last;
  cellstyle
    col = "1" & last = "0" as
      {background = cxFFFFFF borderwidth = 1pt bordercolor = cxFFFFFF
       paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt},
    col = "1" & last = "1" as
      {background = cxFFFFFF borderbottomwidth = 1pt
       paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt},
    col = "2" & last = "0" as
      {background = cxFAFAFA borderwidth = 1pt bordercolor = cxFAFAFA
       paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt},
    col = "2" & last = "1" as
      {background = cxFAFAFA borderbottomwidth = 1pt
       paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt};
  define name;
    header = hname; just = left;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., 3) protectspecialchars = off
    };
    define header hname;
      text &hlabel.; just = center;
      style = {
        bordertopwidth = 0.5pt borderbottomwidth = 1pt
        paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
        background = cxFFFFFF font = (&font., 3)
      };
    end;
  end;
  %do i = 1 %to &K.;
    define c&i.;
      print_headers = off; just = right;
      style = {
        paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
        tagattr = "type:String" font = (&font., 3)
      };
    end;
    define p&i.;
      print_headers = off; just = right;
      style = {
        paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
        tagattr = "type:String" font = (&font., 3)
      };
    end;
    define header l&i.;
      text "&&label&i."; split = "/"; start = c&i.; end = p&i.;
      style = {
        bordertopwidth = 0.5pt borderbottomwidth = 1pt
        paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
        background = cxFFFFFF font = (&font., 3)
      };
    end;
  %end;
  %if "&K." ^= "1" & "&pvalue." = "1" %then %do;
  define pvalue;
    header = xpname; just = right; format = PVALUE5.;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      tagattr = "type:String" font = (&font., 3)
    };
    define header xpname;
      text " ^{style[font_style=Italic]P}-value"; just = center;
      style = {
        bordertopwidth = 0.5pt borderbottomwidth = 1pt
        paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
        background = cxFFFFFF font = (&font., 3)
      };
    end;
  end;
  %end;
  %if "&footer." ^= "0" %then %do;
  define footer footDate;
    text "&footLog."; just = left; force = off;
    style = {
      paddingtop = 5pt paddingbottom = 5pt paddingleft = 7pt paddingright = 7pt
      background = cxFFFFFF borderbottomcolor = white font = (&font., 3)
    };
  end;
  %end;
  define col; print = off; end;
  define last; print = off; end;
end;
run;


/* delete temporary file */
proc datasets lib = work;
  delete _freq_;
run; quit;

%mend tableInit;



%macro tableMake(
  type, var, label, 
  round = 0.1, exact = 1, missing = 0, weight = 0
);
%if "&group." = "" %then %do;
data &data._tmp;
  set &data.;
  _grp_ = 1;
  call symput("K", 1);
run;
%end;
%else %do;
data &data._tmp;
  set &data.;
proc freq data = &data._tmp noprint;
  table &group. / out = _freq_;
data _null_;
  set _freq_;
  call symput("K", trim(cat(_N_)));
run;
%end;

%if "&type." = "cont" %then %contTableMake(&var., &label., &round., 1, 0);;
%if "&type." = "ordr" %then %contTableMake(&var., &label., &round., 0, 1);;
%if "&type." = "disc" %then %discTableMake(&var., &label., &exact., &missing., &weight.);;

proc datasets lib = work;
  delete &data._tmp
    %if "&group." ^= "" %then _freq_;;
run; quit;
%mend tableMake;



%macro contTableMake(var, label, round, mean, med);

%if "&group." = "" %then %let agroup = _grp_;
%else %let agroup = &group.;

proc means data = &data._tmp mean std min median max;
  var &var.;
  class &agroup.;
  output out = _sum_ n = nnum mean = nmean std = nstd min = nmin median = nmed max = nmax
                     q1 = nq1 q3 = nq3;
  format &var. best12.;
data _null_;
  if (log10(&round.)*-1) < 1 then digit = 0;
  else digit = log10(&round.)*-1;
  call symput("rformat1", cat("10.", compress(digit)));
  call symput("rformat2", cat("10.", compress(digit - 1)));
data _sum_;
  length num mean std med range blk $50.;
  set _sum_(where = (_TYPE_ ^= 0));
  num = compress(nnum);
  mean = compress(put(round(nmean, &round.), &rformat1.));
  std = compress(put(round(nstd, &round.), &rformat1.));
  med = compress(put(round(nmed, &round.), &rformat1.));
  range = cat(
    compress(put(round(nmin, &round.), &rformat2.)),
    "-",
    compress(put(round(nmax, &round.), &rformat2.))
  );
  iqr = cat(
    compress(put(round(nq1, &round.), &rformat1.)),
    "-",
    compress(put(round(nq3, &round.), &rformat1.))
  );
  blk = "";
run;
%if "&mean." = "1" %then %do;
  proc transpose data = _sum_ out = _sum0_ prefix = c; var num;
  proc transpose data = _sum_ out = _sum1_ prefix = c; var mean;
  proc transpose data = _sum_ out = _sum2_ prefix = c; var std;
  proc transpose data = _sum_ out = _sum3_ prefix = p; var blk;
  data _sum0_; merge _sum0_ _sum3_; length name $200.; name = '&nbsp;&nbsp;&nbsp;&nbsp;N';
  data _sum1_; merge _sum1_ _sum3_; length name $200.; name = '&nbsp;&nbsp;&nbsp;&nbsp;Mean';
  data _sum2_; merge _sum2_ _sum3_; length name $200.; name = '&nbsp;&nbsp;&nbsp;&nbsp;S.D.';
  data _tbout_; set _sum0_ _sum1_ _sum2_;
%end;
%else %if "&med." = "1" %then %do;
  proc transpose data = _sum_ out = _sum0_ prefix = c; var num;
  proc transpose data = _sum_ out = _sum1_ prefix = c; var med;
  proc transpose data = _sum_ out = _sum2_ prefix = c; var range;
  proc transpose data = _sum_ out = _sum3_ prefix = c; var iqr;
  proc transpose data = _sum_ out = _sum4_ prefix = p; var blk;
  data _sum0_; merge _sum0_ _sum4_; length name $200.; name = '&nbsp;&nbsp;&nbsp;&nbsp;N';
  data _sum1_; merge _sum1_ _sum4_; length name $200.; name = '&nbsp;&nbsp;&nbsp;&nbsp;Median';
  data _sum2_; merge _sum2_ _sum4_; length name $200.; name = '&nbsp;&nbsp;&nbsp;&nbsp;Range';
  data _sum3_; merge _sum3_ _sum4_; length name $200.; name = '&nbsp;&nbsp;&nbsp;&nbsp;IQR';
  data _tbout_; set _sum0_ _sum1_ _sum2_ _sum3_;
%end;
run;

%if "&agroup." ^= "_grp_" & "&pvalue." = "1" & "&mean." = "1" %then %do;
/* Welch type ANOVA */
proc anova data = &data._tmp;
  class &agroup.;
  model &var. = &agroup.;
  means &agroup. / welch;
  ods output Welch = _stat_(where = (source = "&agroup."));
run;
data _stat_;
  length name $200.;
  set _stat_;
  name = &label.;
  rename ProbF = pvalue;
  keep ProbF name;
run;
%end;
%else %if "&agroup." ^= "_grp_" & "&pvalue." = "1" & "&med." = "1" %then %do;
/* Wilcoxon test or Kruskal-Wallis test */
proc npar1way data = &data._tmp wilcoxon;
  class &agroup.;
  var &var.;
  output wilcoxon out = _stat_;
run;
proc freq data = &data. noprint;
  table &agroup. / out = _sum_;
data _null_;
  set _sum_;
  call symput("K", compress(_n_));
run;
data _stat_;
  length name $200.;
  set _stat_;
  name = &label.;
  %if "&K." = "2" %then %let pvar = P2_WIL;
  %else %let pvar = P_KW;
  rename &pvar. = pvalue;
  keep &pvar. name;
run;
%end;
%else %do;
data _stat_;
  length name $200.;
  name = &label.;
run;
%end;

data _tbout_;
  length col $8.;
  set _stat_ _tbout_;
  if "&color." = "2" then col = "1";
  else do;
    col = compress(&color. + 1);
    call symput("color", compress(mod(&color. + 1, 2)));
  end;
  drop _NAME_;
proc append base = &out. data = _tbout_ force;

proc datasets lib = work;
  delete _sum_ _tbout_ _sum0_ _sum1_ _sum2_ _sum3_ _sum4_ _stat_;
run; quit;
%mend contTableMake;



%macro discTableMake(var, label, exact, missing, weight);

%if "&group." = "" %then %let agroup = _grp_;
%else %let agroup = &group.;

data _null_;
  set sashelp.vcolumn;
  if upcase(libname) = upcase("WORK") &
     upcase(memtype) = upcase("DATA") &
     upcase(memname) = upcase("&data.") &
     upcase(name) = upcase("&var.")
  then do;
    if type = "char" then call symput("misschr", "''");
    if type = "num" then call symput("misschr", ".");
  end;
run;

proc freq data = &data._tmp;
  %if "&weight." ^= "0" %then %do;
    weight &weight. / zeros;
  %end;
  table &var. * &agroup. / outpct sparse outexpect
  %if "&missing." = "1" %then %do;
    out = _tab_ missprint missing 
  %end;
  %else %do;
    out = _tab_(where = (&var. ^= &misschr.)) 
  %end;
  %if "&agroup." ^= "_grp_" & "&pvalue." = "1" %then %do;
    %if "&exact." = "1" %then %do;
      chisq fisher;
      output out = _stat_ chisq fisher
    %end;
    %else %do;
      chisq;
      output out = _stat_ chisq
    %end;
  %end;;
proc sort data = _tab_; by &var.;
data _tab_;
  length xpct_col xcount $50.;
  set _tab_;
  xpct_col = cat("(", compress(put(round(pct_col, 0.1), 5.1)), ")");
  xcount = compress(count);
proc transpose data = _tab_ out = _pct_ prefix = p;
  var xpct_col;
  by &var.;
proc transpose data = _tab_ out = _count_ prefix = c;
  var xcount;
  by &var.;
%if "&agroup." ^= "_grp_" & "&pvalue." = "1" & ^%sysfunc(exist(_stat_)) %then %do;
data _stat_;
  length name $200.;
  name = &label.;
  pvalue = .;
%end;
%else %if "&agroup." ^= "_grp_" & "&pvalue." = "1" & %sysfunc(exist(_stat_)) %then %do;
data _null_;
  set _tab_ end = last;
  retain expflg "0";
  if expected = . then;
  else if expected < 5 then expflg = "1";
  if last then call symput("expflg", expflg);
run;
/* Fisher's exact test or Expected value based procedure */
data _stat_;
  length name $200.;
  set _stat_;
  name = &label.;
  %if "&exact." = "1" %then %let pvar = XP2_FISH;
  %else %if "&exact." = "0" %then %let pvar = P_PCHI;
  %else %if "&expflg." = "1" %then %let pvar = XP2_FISH;
  %else %let pvar = P_PCHI;
  rename &pvar. = pvalue;
  keep &pvar. name; 
%end;
%else %do;
data _stat_;
  length name $200.;
  name = &label.;
%end;
data _null_;
  set sashelp.vcolumn;
  if upcase(libname) = upcase("WORK") &
     upcase(memtype) = upcase("DATA") &
     upcase(memname) = upcase("&data.") &
     upcase(name) = upcase("&var.")
  then call symput("varfmt", trim(format));
run;
data _tbout_;
  merge _pct_ _count_;
  by &var.;
  %if "&varfmt." = " " %then name = cat('&nbsp;&nbsp;&nbsp;&nbsp;', trim(cat(&var.)));
  %else name = cat('&nbsp;&nbsp;&nbsp;&nbsp;', trim(put(&var., &varfmt.)));;
data _tbout_;
  set _tbout_(where=(&var. ^= &misschr.)) _tbout_(where=(&var. = &misschr.));
data _tbout_;
  length col $8.;
  set _stat_ _tbout_;
  if "&color." = "2" then col = "1";
  else do;
    col = compress(&color. + 1);
    call symput("color", compress(mod(&color. + 1, 2)));
  end;
  drop _NAME_ _LABEL_ &var.;
proc append base = &out. data = _tbout_ force;
run;
proc datasets lib = work;
  delete _tab_ _tbout_ _pct_ _count_ _stat_;
run; quit;
%mend discTableMake;



%macro render(data); 
data _null_;
  set &data. end = final;
  if final then last = "1";
  else last = "0";
  file print ods = (template = "&data.tmpl");
  put _ods_;
run;
%mend render;



%macro outputInit(file);
ods listing close;
ods results off;

ods tagsets.excelxp file = &file. style = MyJournal;
  title "";
  options missing = " ";
%mend outputInit;



%macro outputMake(out, sheet, title, abscolwidth="40,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7");
ods tagsets.excelxp options(
  absolute_column_width = &abscolwidth.
  row_heights = "22"
  sheet_name = &sheet.
  embedded_titles = "yes"
);
title &title.;
%render(&out.);
%mend outputMake;



%macro outputOut();
  options missing = ".";
ods tagsets.excelxp close;

ods results on;
ods listing;
%mend outputOut;
